@@ -5,31 +5,21 @@
 Link resolver objects
 =====================
 """
-from __future__ import print_function
+
 import codecs
 import gzip
+from io import BytesIO
 import os
+import pickle
 import posixpath
 import re
 import shelve
 import sys
+import urllib.request as urllib_request
+import urllib.parse as urllib_parse
+from urllib.error import HTTPError, URLError
 
-# Try Python 2 first, otherwise load from Python 3
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
-try:
-    import urllib2 as urllib_request
-    from urllib2 import HTTPError, URLError
-    import urlparse as urllib_parse
-except ImportError:
-    import urllib.request as urllib_request
-    import urllib.parse as urllib_parse
-    from urllib.error import HTTPError, URLError
-
-from io import BytesIO
-
+from sphinx.errors import ExtensionError
 from sphinx.search import js_index
 
 from . import sphinx_compatibility
@@ -39,18 +29,19 @@ logger = sphinx_compatibility.getLogger('sphinx-gallery')
 
 
 def _get_data(url):
-    """Helper function to get data over http(s) or from a local file"""
+    """Get data over http(s) or from a local file."""
     if urllib_parse.urlparse(url).scheme in ('http', 'https'):
-        resp = urllib_request.urlopen(url)
+        user_agent = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11'  # noqa: E501
+        headers = {'User-Agent': user_agent}
+        req = urllib_request.Request(url, None, headers)
+        resp = urllib_request.urlopen(req)
         encoding = resp.headers.get('content-encoding', 'plain')
         data = resp.read()
-        if encoding == 'plain':
-            data = data.decode('utf-8')
-        elif encoding == 'gzip':
-            data = BytesIO(data)
-            data = gzip.GzipFile(fileobj=data).read().decode('utf-8')
-        else:
-            raise RuntimeError('unknown encoding')
+        if encoding == 'gzip':
+            data = gzip.GzipFile(fileobj=BytesIO(data)).read()
+        elif encoding != 'plain':
+            raise ExtensionError('unknown encoding %r' % (encoding,))
+        data = data.decode('utf-8')
     else:
         with codecs.open(url, mode='r', encoding='utf-8') as fid:
             data = fid.read()
@@ -60,9 +51,8 @@ def _get_data(url):
 
 def get_data(url, gallery_dir):
     """Persistent dictionary usage to retrieve the search indexes"""
-
     # shelve keys need to be str in python 2
-    if sys.version_info[0] == 2 and isinstance(url, unicode):
+    if sys.version_info[0] == 2 and isinstance(url, str):
         url = url.encode('utf-8')
 
     cached_file = os.path.join(gallery_dir, 'searchindex')
@@ -94,13 +84,16 @@ def parse_sphinx_docopts(index):
 
     pos = index.find('var DOCUMENTATION_OPTIONS')
     if pos < 0:
-        raise ValueError('Documentation options could not be found in index.')
+        raise ExtensionError(
+            'Documentation options could not be found in index.')
     pos = index.find('{', pos)
     if pos < 0:
-        raise ValueError('Documentation options could not be found in index.')
+        raise ExtensionError(
+            'Documentation options could not be found in index.')
     endpos = index.find('};', pos)
     if endpos < 0:
-        raise ValueError('Documentation options could not be found in index.')
+        raise ExtensionError(
+            'Documentation options could not be found in index.')
     block = index[pos + 1:endpos].strip()
     docopts = {}
     for line in block.splitlines():
@@ -143,7 +136,8 @@ class SphinxDocLinkResolver(object):
         package).
     """
 
-    def __init__(self, doc_url, gallery_dir, relative=False):
+    def __init__(self, config, doc_url, gallery_dir, relative=False):
+        self.config = config
         self.doc_url = doc_url
         self.gallery_dir = gallery_dir
         self.relative = relative
@@ -151,22 +145,25 @@ class SphinxDocLinkResolver(object):
 
         if doc_url.startswith(('http://', 'https://')):
             if relative:
-                raise ValueError('Relative links are only supported for local '
-                                 'URLs (doc_url cannot be absolute)')
+                raise ExtensionError(
+                    'Relative links are only supported for local '
+                    'URLs (doc_url cannot be absolute)')
             index_url = doc_url + '/'
             searchindex_url = doc_url + '/searchindex.js'
-            docopts_url = doc_url + '_static/documentation_options.js'
+            docopts_url = doc_url + '/_static/documentation_options.js'
         else:
             index_url = os.path.join(doc_url, 'index.html')
             searchindex_url = os.path.join(doc_url, 'searchindex.js')
-            docopts_url = os.path.join(doc_url, '_static', 'documentation_options.js')
+            docopts_url = os.path.join(
+                doc_url, '_static', 'documentation_options.js')
 
         # detect if we are using relative links on a Windows system
         if (os.name.lower() == 'nt' and
                 not doc_url.startswith(('http://', 'https://'))):
             if not relative:
-                raise ValueError('You have to use relative=True for the local'
-                                 ' package on a Windows system.')
+                raise ExtensionError(
+                    'You have to use relative=True for the local'
+                    ' package on a Windows system.')
             self._is_windows = True
         else:
             self._is_windows = False
@@ -174,9 +171,10 @@ class SphinxDocLinkResolver(object):
         # Download and find documentation options. As of Sphinx 1.7, these
         # options are now kept in a standalone file called
         # 'documentation_options.js'. Since SphinxDocLinkResolver can be called
-        # not only for the documentation which is being built but also ones that
-        # are being referenced, we need to try and get the index page first and
-        # if that doesn't work, check for the documentation_options.js file.
+        # not only for the documentation which is being built but also ones
+        # that are being referenced, we need to try and get the index page
+        # first and if that doesn't work, check for the
+        # documentation_options.js file.
         index = get_data(index_url, gallery_dir)
         if 'var DOCUMENTATION_OPTIONS' in index:
             self._docopts = parse_sphinx_docopts(index)
@@ -188,19 +186,47 @@ class SphinxDocLinkResolver(object):
         sindex = get_data(searchindex_url, gallery_dir)
         self._searchindex = js_index.loads(sindex)
 
-    def _get_link(self, cobj):
-        """Get a valid link, False if not found"""
-
-        fullname = cobj['module_short'] + '.' + cobj['name']
+    def _get_index_match(self, first, second):
         try:
-            value = self._searchindex['objects'][cobj['module_short']]
-            match = value[cobj['name']]
+            match = self._searchindex['objects'][first]
         except KeyError:
-            link = False
+            return None
+        else:
+            if isinstance(match, dict):
+                try:
+                    match = match[second]
+                except KeyError:
+                    return None
+            elif isinstance(match, (list, tuple)):  # Sphinx 5.0.0 dev
+                try:
+                    for item in match:
+                        if item[4] == second:
+                            match = item[:4]
+                            break
+                    else:
+                        return None
+                except Exception:
+                    return None
+        return match
+
+    def _get_link_type(self, cobj, use_full_module=False):
+        """Get a valid link and type_, False if not found."""
+        module_type = 'module_short'
+        if use_full_module:
+            module_type = 'module'
+        first, second = cobj[module_type], cobj['name']
+        match = self._get_index_match(first, second)
+        if match is None and '.' in second:  # possible class attribute
+            first, second = second.split('.', 1)
+            first = '.'.join([cobj['module_short'], first])
+            match = self._get_index_match(first, second)
+        if match is None:
+            link = type_ = None
         else:
             fname_idx = match[0]
             objname_idx = str(match[1])
             anchor = match[3]
+            type_ = self._searchindex['objtypes'][objname_idx]
 
             fname = self._searchindex['filenames'][fname_idx]
             # In 1.5+ Sphinx seems to have changed from .rst.html to only
@@ -213,6 +239,7 @@ class SphinxDocLinkResolver(object):
             else:
                 link = posixpath.join(self.doc_url, fname)
 
+            fullname = '.'.join([first, second])
             if anchor == '':
                 anchor = fullname
             elif anchor == '-':
@@ -221,9 +248,9 @@ class SphinxDocLinkResolver(object):
 
             link = link + '#' + anchor
 
-        return link
+        return link, type_
 
-    def resolve(self, cobj, this_url):
+    def resolve(self, cobj, this_url, return_type=False):
         """Resolve the link to the documentation, returns None if not found
 
         Parameters
@@ -237,25 +264,29 @@ class SphinxDocLinkResolver(object):
         this_url: str
             URL of the current page. Needed to construct relative URLs
             (only used if relative=True in constructor).
+        return_type : bool
+            If True, return the type as well.
 
         Returns
         -------
-        link : str | None
+        link : str or None
             The link (URL) to the documentation.
+        type_ : str
+            The type. Only returned if return_type is True.
         """
         full_name = cobj['module_short'] + '.' + cobj['name']
-        link = self._link_cache.get(full_name, None)
-        if link is None:
+        if full_name not in self._link_cache:
             # we don't have it cached
-            link = self._get_link(cobj)
-            # cache it for the future
-            self._link_cache[full_name] = link
+            use_full_module = False
+            for pattern in self.config['prefer_full_module']:
+                if re.search(pattern, full_name):
+                    use_full_module = True
+                    break
+            self._link_cache[full_name] = self._get_link_type(
+                cobj, use_full_module)
+        link, type_ = self._link_cache[full_name]
 
-        if link is False or link is None:
-            # failed to resolve
-            return None
-
-        if self.relative:
+        if self.relative and link is not None:
             link = os.path.relpath(link, start=this_url)
             if self._is_windows:
                 # replace '\' with '/' so it on the web
@@ -264,7 +295,22 @@ class SphinxDocLinkResolver(object):
             # for some reason, the relative link goes one directory too high up
             link = link[3:]
 
-        return link
+        return (link, type_) if return_type else link
+
+
+def _handle_http_url_error(e, msg='fetching'):
+    if isinstance(e, HTTPError):
+        error_msg = '%s %s: %s (%s)' % (msg, e.url, e.code, e.msg)
+    elif isinstance(e, URLError):
+        error_msg = '%s: %s' % (msg, e.reason)
+    logger.warning('The following %s has occurred %s' % (
+        type(e).__name__, error_msg))
+
+
+def _sanitize_css_class(s):
+    for x in '~!@$%^&*()+=,./\';:"?><[]\\{}|`#':
+        s = s.replace(x, '-')
+    return s
 
 
 def _embed_code_links(app, gallery_conf, gallery_dir):
@@ -276,28 +322,22 @@ def _embed_code_links(app, gallery_conf, gallery_dir):
         try:
             if url is None:
                 doc_resolvers[this_module] = SphinxDocLinkResolver(
+                    app.config.sphinx_gallery_conf,
                     app.builder.outdir, src_gallery_dir, relative=True)
             else:
                 doc_resolvers[this_module] = SphinxDocLinkResolver(
+                    app.config.sphinx_gallery_conf,
                     url, src_gallery_dir)
 
-        except HTTPError as e:
-            logger.warning(
-                'The following HTTP Error has occurred fetching %s: %d %s',
-                e.url, e.code, e.msg)
-        except URLError as e:
-            logger.warning(
-                "Embedding the documentation hyperlinks requires Internet "
-                "access.\nPlease check your network connection.\nUnable to "
-                "continue embedding `%s` links due to a URL Error:\n%s",
-                this_module,
-                str(e.args))
+        except (URLError, HTTPError) as e:
+            _handle_http_url_error(e)
 
     html_gallery_dir = os.path.abspath(os.path.join(app.builder.outdir,
                                                     gallery_dir))
 
     # patterns for replacement
-    link_pattern = ('<a href="%s" title="View documentation for %s">%s</a>')
+    link_pattern = (
+        '<a href="{link}" title="{title}" class="{css_class}">{text}</a>')
     orig_pattern = '<span class="n">%s</span>'
     period = '<span class="o">.</span>'
 
@@ -310,73 +350,100 @@ def _embed_code_links(app, gallery_conf, gallery_dir):
         color='fuchsia', length=len(flat),
         stringify_func=lambda x: os.path.basename(x[1]))
     intersphinx_inv = getattr(app.env, 'intersphinx_named_inventory', dict())
+    builtin_modules = set(intersphinx_inv.get(
+        'python', dict()).get('py:module', dict()).keys())
     for dirpath, fname in iterator:
         full_fname = os.path.join(html_gallery_dir, dirpath, fname)
         subpath = dirpath[len(html_gallery_dir) + 1:]
         pickle_fname = os.path.join(src_gallery_dir, subpath,
                                     fname[:-5] + '_codeobj.pickle')
+        if not os.path.exists(pickle_fname):
+            continue
 
-        if os.path.exists(pickle_fname):
-            # we have a pickle file with the objects to embed links for
-            with open(pickle_fname, 'rb') as fid:
-                example_code_obj = pickle.load(fid)
-            fid.close()
-            str_repl = {}
-            # generate replacement strings with the links
-            for name, cobj in example_code_obj.items():
-                this_module = cobj['module'].split('.')[0]
+        # we have a pickle file with the objects to embed links for
+        with open(pickle_fname, 'rb') as fid:
+            example_code_obj = pickle.load(fid)
+        # generate replacement strings with the links
+        str_repl = {}
+        for name in sorted(example_code_obj):
+            cobjs = example_code_obj[name]
+            # possible names from identify_names, which in turn gets
+            # possibilities from NameFinder.get_mapping
+            link = type_ = None
+            for cobj in cobjs:
+                for modname in (cobj['module_short'], cobj['module']):
+                    this_module = modname.split('.')[0]
+                    cname = cobj['name']
 
-                # Try doc resolvers first
-                link = None
-                if this_module in doc_resolvers:
-                    try:
-                        link = doc_resolvers[this_module].resolve(cobj,
-                                                                  full_fname)
-                    except (HTTPError, URLError) as e:
-                        if isinstance(e, HTTPError):
-                            extra = e.code
+                    # Try doc resolvers first
+                    if this_module in doc_resolvers:
+                        try:
+                            link, type_ = doc_resolvers[this_module].resolve(
+                                cobj, full_fname, return_type=True)
+                        except (HTTPError, URLError) as e:
+                            _handle_http_url_error(
+                                e, msg='resolving %s.%s' % (modname, cname))
+
+                    # next try intersphinx
+                    if this_module == modname == 'builtins':
+                        this_module = 'python'
+                    elif modname in builtin_modules:
+                        this_module = 'python'
+                    if link is None and this_module in intersphinx_inv:
+                        inv = intersphinx_inv[this_module]
+                        if modname == 'builtins':
+                            want = cname
                         else:
-                            extra = e.reason
-                        logger.warning("Error resolving %s.%s: %r (%s)",
-                                       cobj['module'], cobj['name'], e, extra)
-                        link = None
+                            want = '%s.%s' % (modname, cname)
+                        for key, value in inv.items():
+                            # only python domain
+                            if key.startswith('py') and want in value:
+                                link = value[want][2]
+                                type_ = key
+                                break
 
-                # next try intersphinx
-                if link is None and this_module in intersphinx_inv:
-                    inv = app.env.intersphinx_named_inventory[this_module]
-                    want = '%s.%s' % (cobj['module'], cobj['name'])
-                    for value in inv.values():
-                        if want in value:
-                            link = value[want][2]
-                            break
+                    # differentiate classes from instances
+                    is_instance = (type_ is not None and
+                                   'py:class' in type_ and
+                                   not cobj['is_class'])
+
+                    if link is not None:
+                        # Add CSS classes
+                        name_html = period.join(orig_pattern % part
+                                                for part in name.split('.'))
+                        full_function_name = '%s.%s' % (modname, cname)
+                        css_class = ("sphx-glr-backref-module-" +
+                                     _sanitize_css_class(modname))
+                        if type_ is not None:
+                            css_class += (" sphx-glr-backref-type-" +
+                                          _sanitize_css_class(type_))
+                        if is_instance:
+                            css_class += " sphx-glr-backref-instance"
+                        str_repl[name_html] = link_pattern.format(
+                            link=link, title=full_function_name,
+                            css_class=css_class, text=name_html)
+                        break  # loop over possible module names
 
                 if link is not None:
-                    parts = name.split('.')
-                    name_html = period.join(orig_pattern % part
-                                            for part in parts)
-                    full_function_name = '%s.%s' % (
-                        cobj['module'], cobj['name'])
-                    str_repl[name_html] = link_pattern % (
-                        link, full_function_name, name_html)
+                    break  # loop over cobjs
 
-            # do the replacement in the html file
+        # do the replacement in the html file
 
-            # ensure greediness
-            names = sorted(str_repl, key=len, reverse=True)
-            regex_str = '|'.join(re.escape(name) for name in names)
-            regex = re.compile(regex_str)
+        # ensure greediness
+        names = sorted(str_repl, key=len, reverse=True)
+        regex_str = '|'.join(re.escape(name) for name in names)
+        regex = re.compile(regex_str)
 
-            def substitute_link(match):
-                return str_repl[match.group()]
+        def substitute_link(match):
+            return str_repl[match.group()]
 
-            if len(str_repl) > 0:
-                with open(full_fname, 'rb') as fid:
-                    lines_in = fid.readlines()
-                with open(full_fname, 'wb') as fid:
-                    for line in lines_in:
-                        line = line.decode('utf-8')
-                        line = regex.sub(substitute_link, line)
-                        fid.write(line.encode('utf-8'))
+        if len(str_repl) > 0:
+            with codecs.open(full_fname, 'r', 'utf-8') as fid:
+                lines_in = fid.readlines()
+            with codecs.open(full_fname, 'w', 'utf-8') as fid:
+                for line in lines_in:
+                    line_out = regex.sub(substitute_link, line)
+                    fid.write(line_out)
 
 
 def embed_code_links(app, exception):
@@ -384,11 +451,7 @@ def embed_code_links(app, exception):
     if exception is not None:
         return
 
-    # No need to waste time embedding hyperlinks when not running the examples
-    # XXX: also at the time of writing this fixes make html-noplot
-    # for some reason I don't fully understand
-    if not app.builder.config.plot_gallery:
-        return
+    gallery_conf = app.config.sphinx_gallery_conf
 
     # XXX: Whitelist of builders for which it makes sense to embed
     # hyperlinks inside the example html. Note that the link embedding
@@ -400,8 +463,6 @@ def embed_code_links(app, exception):
 
     logger.info('embedding documentation hyperlinks...', color='white')
 
-    gallery_conf = app.config.sphinx_gallery_conf
-
     gallery_dirs = gallery_conf['gallery_dirs']
     if not isinstance(gallery_dirs, list):
         gallery_dirs = [gallery_dirs]
