@@ -12,36 +12,38 @@ from __future__ import absolute_import, division, print_function
 import os
 import zipfile
 
+from .utils import _replace_md5
+
 CODE_DOWNLOAD = """
 .. _sphx_glr_download_{3}:
 
-\n.. only :: html
+.. only:: html
 
- .. container:: sphx-glr-footer
-    :class: sphx-glr-footer-example
+  .. container:: sphx-glr-footer sphx-glr-footer-example
 
 {2}
-\n  .. container:: sphx-glr-download
+    .. container:: sphx-glr-download sphx-glr-download-python
 
-     :download:`Download Python source code: {0} <{0}>`\n
+      :download:`Download Python source code: {0} <{0}>`
 
-\n  .. container:: sphx-glr-download
+    .. container:: sphx-glr-download sphx-glr-download-jupyter
 
-     :download:`Download Jupyter notebook: {1} <{1}>`\n"""
+      :download:`Download Jupyter notebook: {1} <{1}>`
+"""
 
 CODE_ZIP_DOWNLOAD = """
-\n.. only :: html
+.. only:: html
 
- .. container:: sphx-glr-footer
-    :class: sphx-glr-footer-gallery
+  .. container:: sphx-glr-footer sphx-glr-footer-gallery
 
-\n  .. container:: sphx-glr-download
+    .. container:: sphx-glr-download sphx-glr-download-python
 
-    :download:`Download all examples in Python source code: {0} </{1}>`\n
+      :download:`Download all examples in Python source code: {0} </{1}>`
 
-\n  .. container:: sphx-glr-download
+    .. container:: sphx-glr-download sphx-glr-download-jupyter
 
-    :download:`Download all examples in Jupyter notebooks: {2} </{3}>`\n"""
+      :download:`Download all examples in Jupyter notebooks: {2} </{3}>`
+"""
 
 
 def python_zip(file_list, gallery_path, extension='.py'):
@@ -49,7 +51,7 @@ def python_zip(file_list, gallery_path, extension='.py'):
 
     Parameters
     ----------
-    file_list : list of strings
+    file_list : list
         Holds all the file names to be included in zip file
     gallery_path : str
         path to where the zipfile is stored
@@ -67,13 +69,12 @@ def python_zip(file_list, gallery_path, extension='.py'):
     zipname = os.path.basename(os.path.normpath(gallery_path))
     zipname += '_python' if extension == '.py' else '_jupyter'
     zipname = os.path.join(gallery_path, zipname + '.zip')
-
-    zipf = zipfile.ZipFile(zipname, mode='w')
-    for fname in file_list:
-        file_src = os.path.splitext(fname)[0] + extension
-        zipf.write(file_src, os.path.relpath(file_src, gallery_path))
-    zipf.close()
-
+    zipname_new = zipname + '.new'
+    with zipfile.ZipFile(zipname_new, mode='w') as zipf:
+        for fname in file_list:
+            file_src = os.path.splitext(fname)[0] + extension
+            zipf.write(file_src, os.path.relpath(file_src, gallery_path))
+    _replace_md5(zipname_new)
     return zipname
 
 
@@ -94,7 +95,7 @@ def list_downloadable_sources(target_dir):
             if fname.endswith('.py')]
 
 
-def generate_zipfiles(gallery_dir):
+def generate_zipfiles(gallery_dir, src_dir):
     """
     Collects all Python source files and Jupyter notebooks in
     gallery_dir and makes zipfiles of them
@@ -103,6 +104,8 @@ def generate_zipfiles(gallery_dir):
     ----------
     gallery_dir : str
         path of the gallery to collect downloadable sources
+    src_dir : str
+        The build source directory. Needed to make the RST paths relative.
 
     Return
     ------
@@ -120,6 +123,7 @@ def generate_zipfiles(gallery_dir):
     jy_zipfile = python_zip(listdir, gallery_dir, ".ipynb")
 
     def rst_path(filepath):
+        filepath = os.path.relpath(filepath, os.path.normpath(src_dir))
         return filepath.replace(os.sep, '/')
 
     dw_rst = CODE_ZIP_DOWNLOAD.format(os.path.basename(py_zipfile),
