@@ -33,11 +33,11 @@ from .docs_resolv import embed_code_links
 from .downloads import generate_zipfiles
 from .sorting import NumberOfCodeLinesSortKey
 from .binder import copy_binder_files, check_binder_conf
-from .directives import MiniGallery
+from .directives import MiniGallery, ImageSg, imagesg_addnode
 
 
-_KNOWN_CSS = ('gallery', 'gallery-binder', 'gallery-dataframe',
-              'gallery-rendered-html')
+_KNOWN_CSS = ('sg_gallery', 'sg_gallery-binder', 'sg_gallery-dataframe',
+              'sg_gallery-rendered-html')
 
 
 class DefaultResetArgv:
@@ -58,15 +58,16 @@ DEFAULT_GALLERY_CONF = {
     'gallery_dirs': 'auto_examples',
     'backreferences_dir': None,
     'doc_module': (),
+    'exclude_implicit_doc': {},
     'reference_url': {},
     'capture_repr': ('_repr_html_', '__repr__'),
     'ignore_repr_types': r'',
     # Build options
     # -------------
-    # We use a string for 'plot_gallery' rather than simply the Python boolean
-    # `True` as it avoids a warning about unicode when controlling this value
-    # via the command line switches of sphinx-build
-    'plot_gallery': 'True',
+    # 'plot_gallery' also accepts strings that evaluate to a bool, e.g. "True",
+    # "False", "1", "0" so that they can be easily set via command line
+    # switches of sphinx-build
+    'plot_gallery': True,
     'download_all_examples': True,
     'abort_on_example_error': False,
     'only_warn_on_example_error': False,
@@ -78,9 +79,11 @@ DEFAULT_GALLERY_CONF = {
     'thumbnail_size': (400, 280),  # Default CSS does 0.4 scaling (160, 112)
     'min_reported_time': 0,
     'binder': {},
+    'promote_jupyter_magic': False,
     'image_scrapers': ('matplotlib',),
     'compress_images': (),
     'reset_modules': ('matplotlib', 'seaborn'),
+    'reset_modules_order': 'before',
     'first_notebook_cell': '%matplotlib inline',
     'last_notebook_cell': None,
     'notebook_images': False,
@@ -93,8 +96,11 @@ DEFAULT_GALLERY_CONF = {
     'inspect_global_variables': True,
     'css': _KNOWN_CSS,
     'matplotlib_animations': False,
+    'image_srcset': [],
     'default_thumb_file': None,
     'line_numbers': False,
+    'nested_sections': True,
+    'prefer_full_module': [],
 }
 
 logger = sphinx_compatibility.getLogger('sphinx-gallery')
@@ -127,6 +133,17 @@ def parse_config(app, check_keys=True):
     return gallery_conf
 
 
+def _update_gallery_conf(gallery_conf):
+    """Update gallery config.
+
+    This is separate function for better testability.
+    """
+    # prepare regex for exclusions from implicit documentation
+    exclude_regex = re.compile('|'.join(gallery_conf['exclude_implicit_doc']))\
+        if gallery_conf['exclude_implicit_doc'] else False
+    gallery_conf['exclude_implicit_doc_regex'] = exclude_regex
+
+
 def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
                            abort_on_example_error, lang='python',
                            builder_name='html', app=None, check_keys=True):
@@ -156,7 +173,7 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
     gallery_conf.update(abort_on_example_error=abort_on_example_error)
     # XXX anything that can only be a bool (rather than str) should probably be
     # evaluated this way as it allows setting via -D on the command line
-    for key in ('run_stale_examples',):
+    for key in ('promote_jupyter_magic', 'run_stale_examples',):
         gallery_conf[key] = _bool_eval(gallery_conf[key])
     gallery_conf['src_dir'] = src_dir
     gallery_conf['app'] = app
@@ -286,6 +303,14 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
                               % (resetter,))
     gallery_conf['reset_modules'] = tuple(resetters)
 
+    if not isinstance(gallery_conf['reset_modules_order'], str):
+        raise ConfigError('reset_modules_order must be a str, '
+                          'got %r' % gallery_conf['reset_modules_order'])
+    if gallery_conf['reset_modules_order'] not in ['before', 'after', 'both']:
+        raise ConfigError("reset_modules_order must be in"
+                          "['before', 'after', 'both'], "
+                          'got %r' % gallery_conf['reset_modules_order'])
+
     lang = lang if lang in ('python', 'python3', 'default') else 'python'
     gallery_conf['lang'] = lang
     del resetters
@@ -353,6 +378,8 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
         if gallery_conf['app'] is not None:  # can be None in testing
             gallery_conf['app'].add_css_file(css + '.css')
 
+    _update_gallery_conf(gallery_conf)
+
     return gallery_conf
 
 
@@ -410,8 +437,27 @@ def generate_gallery_rst(app):
     """Generate the Main examples gallery reStructuredText
 
     Start the Sphinx-Gallery configuration and recursively scan the examples
-    directories in order to populate the examples gallery
+    directories in order to populate the examples gallery.
+
+    We create a 2-level nested structure by iterating through every
+    sibling folder of the current index file.
+    In each of these folders, we look for a section index file,
+    for which we generate a toctree pointing to sibling scripts.
+    Then, we append the content of this section index file
+    to the current index file,
+    after we remove toctree (to keep a clean nested structure)
+    and sphinx tags (to prevent tag duplication)
+    Eventually, we create a toctree in the current index file
+    which points to section index files.
     """
+    blocked_builder = ['dirhtml']
+    if app.builder.name in blocked_builder:
+        msg = (
+            f'Builder is set to {repr(app.builder.name)}. sphinx_gallery does '
+            f'not work for any of the following: {[blocked_builder]}'
+        )
+        raise ConfigError(msg)
+
     logger.info('generating gallery...', color='white')
     gallery_conf = parse_config(app)
 
@@ -429,41 +475,124 @@ def generate_gallery_rst(app):
 
     for examples_dir, gallery_dir in workdirs:
 
-        examples_dir = os.path.join(app.builder.srcdir, examples_dir)
-        gallery_dir = os.path.join(app.builder.srcdir, gallery_dir)
+        examples_dir_abs_path = os.path.join(app.builder.srcdir, examples_dir)
+        gallery_dir_abs_path = os.path.join(app.builder.srcdir, gallery_dir)
 
-        # Here we don't use an os.walk, but we recurse only twice: flat is
-        # better than nested.
-        this_fhindex, this_costs = generate_dir_rst(
-            examples_dir, gallery_dir, gallery_conf, seen_backrefs)
+        # Create section rst files and fetch content which will
+        # be added to current index file. This only includes content
+        # from files located in the root folder of the current gallery
+        # (ie not in subfolders)
+        (
+            _,
+            this_content,
+            this_costs,
+            this_toctree_items,
+        ) = generate_dir_rst(
+            examples_dir_abs_path,
+            gallery_dir_abs_path,
+            gallery_conf,
+            seen_backrefs,
+            include_toctree=False,
+        )
 
         costs += this_costs
-        write_computation_times(gallery_conf, gallery_dir, this_costs)
+        write_computation_times(gallery_conf, gallery_dir_abs_path, this_costs)
 
-        # we create an index.rst with all examples
-        index_rst_new = os.path.join(gallery_dir, 'index.rst.new')
+        # We create an index.rst with all examples
+        # (this will overwrite the rst file generated by the previous call
+        # to generate_dir_rst)
+        index_rst_new = os.path.join(gallery_dir_abs_path, 'index.rst.new')
         with codecs.open(index_rst_new, 'w', encoding='utf-8') as fhindex:
             # :orphan: to suppress "not included in TOCTREE" sphinx warnings
-            fhindex.write(":orphan:\n\n" + this_fhindex)
+            fhindex.write(":orphan:\n\n" + this_content)
+
+            # Write toctree with gallery items from gallery root folder
+            if len(this_toctree_items) > 0:
+                this_toctree = """
+.. toctree::
+   :hidden:
+
+   %s\n
+""" % "\n   ".join(this_toctree_items)
+                fhindex.write(this_toctree)
+
+            # list all paths to subsection index files in this array
+            subsection_index_files = []
 
             for subsection in get_subsections(
-                    app.builder.srcdir, examples_dir, gallery_conf):
-                src_dir = os.path.join(examples_dir, subsection)
-                target_dir = os.path.join(gallery_dir, subsection)
-                this_fhindex, this_costs = \
-                    generate_dir_rst(src_dir, target_dir, gallery_conf,
-                                     seen_backrefs)
-                fhindex.write(this_fhindex)
-                costs += this_costs
-                write_computation_times(gallery_conf, target_dir, this_costs)
+                    app.builder.srcdir, examples_dir_abs_path, gallery_conf):
+                src_dir = os.path.join(examples_dir_abs_path, subsection)
+                target_dir = os.path.join(gallery_dir_abs_path, subsection)
+                subsection_index_files.append(
+                    '/'.join([
+                        '', gallery_dir, subsection, 'index.rst'
+                    ])
+                )
+
+                (
+                    subsection_index_path,
+                    subsection_index_content,
+                    subsection_costs,
+                    subsection_toctree_filenames,
+                ) = generate_dir_rst(
+                    src_dir, target_dir, gallery_conf, seen_backrefs
+                )
+
+                # Filter out tags from subsection content
+                # to prevent tag duplication across the documentation
+                tag_regex = r"^\.\.(\s+)\_(.+)\:(\s*)$"
+                subsection_index_content = "\n".join([
+                    line for line in subsection_index_content.splitlines()
+                    if re.match(tag_regex, line) is None
+                ] + [''])
+
+                fhindex.write(subsection_index_content)
+
+                # Write subsection toctree in main file
+                # only if nested_sections is False or None
+                if not gallery_conf["nested_sections"]:
+                    if len(subsection_toctree_filenames) > 0:
+                        subsection_index_toctree = """
+.. toctree::
+   :hidden:
+
+   %s\n
+""" % "\n   ".join(subsection_toctree_filenames)
+                    fhindex.write(subsection_index_toctree)
+                # Otherwise, a new index.rst.new file should
+                # have been created and it needs to be parsed
+                else:
+                    _replace_md5(subsection_index_path, mode='t')
+
+                costs += subsection_costs
+                write_computation_times(
+                    gallery_conf, target_dir, subsection_costs
+                )
+
+            # generate toctree with subsections
+            if gallery_conf["nested_sections"] is True:
+                subsections_toctree = """
+.. toctree::
+   :hidden:
+   :includehidden:
+
+   %s\n
+""" % "\n   ".join(subsection_index_files)
+# """ % "\n   ".join(this_toctree_items + subsection_index_files)
+
+                # add toctree to file only if there are subsections
+                if len(subsection_index_files) > 0:
+                    fhindex.write(subsections_toctree)
 
             if gallery_conf['download_all_examples']:
                 download_fhindex = generate_zipfiles(
-                    gallery_dir, app.builder.srcdir)
+                    gallery_dir_abs_path, app.builder.srcdir
+                )
                 fhindex.write(download_fhindex)
 
             if (app.config.sphinx_gallery_conf['show_signature']):
                 fhindex.write(SPHX_GLR_SIG)
+
         _replace_md5(index_rst_new, mode='t')
     _finalize_backreferences(seen_backrefs, gallery_conf)
 
@@ -785,6 +914,9 @@ def setup(app):
 
     # Add the custom directive
     app.add_directive('minigallery', MiniGallery)
+    app.add_directive("image-sg", ImageSg)
+
+    imagesg_addnode(app)
 
     app.connect('builder-inited', generate_gallery_rst)
     app.connect('build-finished', copy_binder_files)
