@@ -45,7 +45,8 @@ from .backreferences import (_write_backreferences, _thumbnail_div,
                              identify_names)
 from .downloads import CODE_DOWNLOAD
 from .py_source_parser import (split_code_and_text_blocks,
-                               remove_config_comments)
+                               remove_config_comments,
+                               remove_ignore_blocks)
 
 from .notebook import jupyter_notebook, save_notebook
 from .binder import check_binder_conf, gen_binder_rst
@@ -105,6 +106,10 @@ class _LoggingTee(object):
             self.logger.verbose('%s', self.logger_buffer)
             self.logger_buffer = ''
 
+    # Needed to work with Abseil
+    def close(self):
+        pass
+
     # When called from a local terminal seaborn needs it in Python3
     def isatty(self):
         return self.output.isatty()
@@ -140,8 +145,6 @@ SINGLE_IMAGE = """
 
 CODE_OUTPUT = """.. rst-class:: sphx-glr-script-out
 
- Out:
-
  .. code-block:: none
 
 {0}\n"""
@@ -162,7 +165,7 @@ SPHX_GLR_SIG = """\n
 """
 
 # Header used to include raw html
-html_header = """.. raw:: html
+HTML_HEADER = """.. raw:: html
 
     <div class="output_subarea output_html rendered_html output_result">
 {0}
@@ -345,17 +348,72 @@ def _get_readme(dir_, gallery_conf, raise_error=True):
     return None
 
 
-def generate_dir_rst(src_dir, target_dir, gallery_conf, seen_backrefs):
-    """Generate the gallery reStructuredText for an example directory."""
+THUMBNAIL_PARENT_DIV = """
+.. raw:: html
+
+    <div class="sphx-glr-thumbnails">
+
+"""
+
+THUMBNAIL_PARENT_DIV_CLOSE = """
+.. raw:: html
+
+    </div>
+
+"""
+
+
+def generate_dir_rst(
+    src_dir,
+    target_dir,
+    gallery_conf,
+    seen_backrefs,
+    include_toctree=True,
+):
+    """Generate the gallery reStructuredText for an example directory.
+
+    Parameters
+    ----------
+    src_dir: str,
+        Path to example directory containing python files
+        and possibly sub categories
+    target_dir: str,
+        Path where parsed examples (rst, python files, etc)
+        will be outputed
+    gallery_conf : Dict[str, Any]
+        Gallery configurations.
+    seen_backrefs: set,
+        Back references encountered when parsing this gallery
+        will be stored in this set.
+    include_toctree: bool,
+        Whether or not toctree should be included
+        in generated rst file.
+        Default = True.
+
+    Returns
+    -------
+    index_path: str,
+        Path to index rst file presenting the current example gallery
+    index_content: str,
+        Content which will be written to the index rst file
+        presenting the current example gallery
+    costs: list,
+        List of costs for building each element of the gallery
+    toctree_items: list,
+        List of files included in toctree
+        (independent of include_toctree's value)
+    """
     head_ref = os.path.relpath(target_dir, gallery_conf['src_dir'])
-    fhindex = """\n\n.. _sphx_glr_{0}:\n\n""".format(
-        head_ref.replace(os.path.sep, '_'))
+
+    subsection_index_content = ""
+    subsection_readme_fname = _get_readme(src_dir, gallery_conf)
+
+    with codecs.open(subsection_readme_fname, 'r', encoding='utf-8') as fid:
+        subsection_readme_content = fid.read()
+        subsection_index_content += subsection_readme_content
 
-    fname = _get_readme(src_dir, gallery_conf)
-    with codecs.open(fname, 'r', encoding='utf-8') as fid:
-        fhindex += fid.read()
     # Add empty lines to avoid bug in issue #165
-    fhindex += "\n\n"
+    subsection_index_content += "\n\n"
 
     if not os.path.exists(target_dir):
         os.makedirs(target_dir)
@@ -370,8 +428,14 @@ def generate_dir_rst(src_dir, target_dir, gallery_conf, seen_backrefs):
     # sort them
     sorted_listdir = sorted(
         listdir, key=gallery_conf['within_subsection_order'](src_dir))
+
+    # Add div containing all thumbnails;
+    # this is helpful for controlling grid or flexbox behaviours
+    subsection_index_content += THUMBNAIL_PARENT_DIV
+
     entries_text = []
     costs = []
+    subsection_toctree_filenames = []
     build_target_dir = os.path.relpath(target_dir, gallery_conf['src_dir'])
     iterator = sphinx_compatibility.status_iterator(
         sorted_listdir,
@@ -382,23 +446,57 @@ def generate_dir_rst(src_dir, target_dir, gallery_conf, seen_backrefs):
             fname, target_dir, src_dir, gallery_conf, seen_backrefs)
         src_file = os.path.normpath(os.path.join(src_dir, fname))
         costs.append((cost, src_file))
-        this_entry = _thumbnail_div(target_dir, gallery_conf['src_dir'],
-                                    fname, intro, title) + """
+        gallery_item_filename = os.path.join(
+            build_target_dir,
+            fname[:-3]
+        ).replace(os.sep, '/')
+        this_entry = _thumbnail_div(
+            target_dir, gallery_conf['src_dir'], fname, intro, title
+        )
+        entries_text.append(this_entry)
+        subsection_toctree_filenames.append("/" + gallery_item_filename)
 
+    for entry_text in entries_text:
+        subsection_index_content += entry_text
+
+    # Close thumbnail parent div
+    subsection_index_content += THUMBNAIL_PARENT_DIV_CLOSE
+
+    # Write subsection index file
+    # only if nested_sections is True
+    subsection_index_path = None
+    if gallery_conf["nested_sections"] is True:
+        subsection_index_path = os.path.join(target_dir, 'index.rst.new')
+        with codecs.open(subsection_index_path, 'w', encoding='utf-8') as (
+            findex
+        ):
+            findex.write("""\n\n.. _sphx_glr_{0}:\n\n""".format(
+                head_ref.replace(os.path.sep, '_')
+            ))
+            findex.write(subsection_index_content)
+
+            # Create toctree for index file
+            # with all gallery items which belong to current subsection
+            # and add it to generated index rst file if need be.
+            # Toctree cannot be empty
+            # and won't be added if include_toctree is false
+            # (this is useful when generating the example gallery's main
+            # index rst file, which should contain only one toctree)
+            if len(subsection_toctree_filenames) > 0 and include_toctree:
+                subsection_index_toctree = """
 .. toctree::
    :hidden:
 
-   /%s\n""" % os.path.join(build_target_dir, fname[:-3]).replace(os.sep, '/')
-        entries_text.append(this_entry)
+   %s\n
+""" % "\n   ".join(subsection_toctree_filenames)
+                findex.write(subsection_index_toctree)
 
-    for entry_text in entries_text:
-        fhindex += entry_text
-
-    # clear at the end of the section
-    fhindex += """.. raw:: html\n
-    <div class="sphx-glr-clear"></div>\n\n"""
-
-    return fhindex, costs
+    return (
+        subsection_index_path,
+        subsection_index_content,
+        costs,
+        subsection_toctree_filenames,
+    )
 
 
 def handle_exception(exc_info, src_file, script_vars, gallery_conf):
@@ -542,139 +640,14 @@ def _get_memory_base(gallery_conf):
     return memory_base
 
 
-def execute_code_block(compiler, block, example_globals,
-                       script_vars, gallery_conf):
-    """Execute the code block of the example file."""
-    if example_globals is None:  # testing shortcut
-        example_globals = script_vars['fake_main'].__dict__
-    blabel, bcontent, lineno = block
-    # If example is not suitable to run, skip executing its blocks
-    if not script_vars['execute_script'] or blabel == 'text':
-        return ''
-
-    cwd = os.getcwd()
-    # Redirect output to stdout and
-
-    src_file = script_vars['src_file']
-    logging_tee = _check_reset_logging_tee(src_file)
-    assert isinstance(logging_tee, _LoggingTee)
-
-    # First cd in the original example dir, so that any file
-    # created by the example get created in this directory
-
-    os.chdir(os.path.dirname(src_file))
-
-    sys_path = copy.deepcopy(sys.path)
-    sys.path.append(os.getcwd())
-
-    # Save figures unless there is a `sphinx_gallery_defer_figures` flag
-    match = re.search(r'^[\ \t]*#\s*sphinx_gallery_defer_figures[\ \t]*\n?',
-                      bcontent, re.MULTILINE)
-    need_save_figures = match is None
-
-    try:
-        dont_inherit = 1
-        if sys.version_info >= (3, 8):
-            ast_Module = partial(ast.Module, type_ignores=[])
-        else:
-            ast_Module = ast.Module
-        code_ast = ast_Module([bcontent])
-        flags = ast.PyCF_ONLY_AST | compiler.flags
-        code_ast = compile(bcontent, src_file, 'exec', flags, dont_inherit)
-        ast.increment_lineno(code_ast, lineno - 1)
-        # capture output if last line is expression
-        is_last_expr = False
-        if len(code_ast.body) and isinstance(code_ast.body[-1], ast.Expr):
-            is_last_expr = True
-            last_val = code_ast.body.pop().value
-            # exec body minus last expression
-            mem_body, _ = gallery_conf['call_memory'](
-                _exec_once(
-                    compiler(code_ast, src_file, 'exec'),
-                    script_vars['fake_main']))
-            # exec last expression, made into assignment
-            body = [ast.Assign(
-                targets=[ast.Name(id='___', ctx=ast.Store())], value=last_val)]
-            last_val_ast = ast_Module(body=body)
-            ast.fix_missing_locations(last_val_ast)
-            mem_last, _ = gallery_conf['call_memory'](
-                _exec_once(
-                    compiler(last_val_ast, src_file, 'exec'),
-                    script_vars['fake_main']))
-            # capture the assigned variable
-            ___ = example_globals['___']
-            mem_max = max(mem_body, mem_last)
-        else:
-            mem_max, _ = gallery_conf['call_memory'](
-                _exec_once(
-                    compiler(code_ast, src_file, 'exec'),
-                    script_vars['fake_main']))
-        script_vars['memory_delta'].append(mem_max)
-        # This should be inside the try block, e.g., in case of a savefig error
-        logging_tee.restore_std()
-        if need_save_figures:
-            need_save_figures = False
-            images_rst = save_figures(block, script_vars, gallery_conf)
-        else:
-            images_rst = u''
-    except Exception:
-        logging_tee.restore_std()
-        except_rst = handle_exception(sys.exc_info(), src_file, script_vars,
-                                      gallery_conf)
-        code_output = u"\n{0}\n\n\n\n".format(except_rst)
-        # still call this even though we won't use the images so that
-        # figures are closed
-        if need_save_figures:
-            save_figures(block, script_vars, gallery_conf)
+def _ast_module():
+    """Get ast.Module function, dealing with:
+    https://bugs.python.org/issue35894"""
+    if sys.version_info >= (3, 8):
+        ast_Module = partial(ast.Module, type_ignores=[])
     else:
-        sys.path = sys_path
-        os.chdir(cwd)
-
-        last_repr = None
-        repr_meth = None
-        if is_last_expr:
-            if gallery_conf['ignore_repr_types']:
-                ignore_repr = re.search(
-                    gallery_conf['ignore_repr_types'], str(type(___))
-                )
-            else:
-                ignore_repr = False
-            if gallery_conf['capture_repr'] != () and not ignore_repr:
-                for meth in gallery_conf['capture_repr']:
-                    try:
-                        last_repr = getattr(___, meth)()
-                        # for case when last statement is print()
-                        if last_repr is None or last_repr == 'None':
-                            repr_meth = None
-                        else:
-                            repr_meth = meth
-                    except Exception:
-                        pass
-                    else:
-                        if isinstance(last_repr, str):
-                            break
-        captured_std = logging_tee.output.getvalue().expandtabs()
-        # normal string output
-        if repr_meth in ['__repr__', '__str__'] and last_repr:
-            captured_std = u"{0}\n{1}".format(captured_std, last_repr)
-        if captured_std and not captured_std.isspace():
-            captured_std = CODE_OUTPUT.format(indent(captured_std, u' ' * 4))
-        else:
-            captured_std = ''
-        # give html output its own header
-        if repr_meth == '_repr_html_':
-            captured_html = html_header.format(indent(last_repr, u' ' * 4))
-        else:
-            captured_html = ''
-        code_output = u"\n{0}\n\n{1}\n{2}\n\n".format(
-            images_rst, captured_std, captured_html)
-
-    finally:
-        os.chdir(cwd)
-        sys.path = sys_path
-        logging_tee.restore_std()
-
-    return code_output
+        ast_Module = ast.Module
+    return ast_Module
 
 
 def _check_reset_logging_tee(src_file):
@@ -688,6 +661,205 @@ def _check_reset_logging_tee(src_file):
     return logging_tee
 
 
+def _exec_and_get_memory(compiler, ast_Module, code_ast, gallery_conf,
+                         script_vars):
+    """Execute ast, capturing output if last line is expression and get max
+    memory usage."""
+    src_file = script_vars['src_file']
+    # capture output if last line is expression
+    is_last_expr = False
+    if len(code_ast.body) and isinstance(code_ast.body[-1], ast.Expr):
+        is_last_expr = True
+        last_val = code_ast.body.pop().value
+        # exec body minus last expression
+        mem_body, _ = gallery_conf['call_memory'](
+            _exec_once(
+                compiler(code_ast, src_file, 'exec'),
+                script_vars['fake_main']))
+        # exec last expression, made into assignment
+        body = [ast.Assign(
+            targets=[ast.Name(id='___', ctx=ast.Store())], value=last_val)]
+        last_val_ast = ast_Module(body=body)
+        ast.fix_missing_locations(last_val_ast)
+        mem_last, _ = gallery_conf['call_memory'](
+            _exec_once(
+                compiler(last_val_ast, src_file, 'exec'),
+                script_vars['fake_main']))
+        mem_max = max(mem_body, mem_last)
+    else:
+        mem_max, _ = gallery_conf['call_memory'](
+            _exec_once(
+                compiler(code_ast, src_file, 'exec'),
+                script_vars['fake_main']))
+    return is_last_expr, mem_max
+
+
+def _get_last_repr(capture_repr, ___):
+    """Get a repr of the last expression, using first method in 'capture_repr'
+    available for the last expression."""
+    for meth in capture_repr:
+        try:
+            last_repr = getattr(___, meth)()
+            # for case when last statement is print()
+            if last_repr is None or last_repr == 'None':
+                repr_meth = None
+            else:
+                repr_meth = meth
+        except Exception:
+            last_repr = None
+            repr_meth = None
+        else:
+            if isinstance(last_repr, str):
+                break
+    return last_repr, repr_meth
+
+
+def _get_code_output(is_last_expr, example_globals, gallery_conf, logging_tee,
+                     images_rst, file_conf):
+    """Obtain standard output and html output in rST."""
+    last_repr = None
+    repr_meth = None
+    if is_last_expr:
+        # capture the last repr variable
+        ___ = example_globals['___']
+        ignore_repr = False
+        if gallery_conf['ignore_repr_types']:
+            ignore_repr = re.search(
+                gallery_conf['ignore_repr_types'], str(type(___))
+            )
+        capture_repr = file_conf.get('capture_repr',
+                                     gallery_conf['capture_repr'])
+        if capture_repr != () and not ignore_repr:
+            last_repr, repr_meth = _get_last_repr(capture_repr, ___)
+
+    captured_std = logging_tee.output.getvalue().expandtabs()
+    # normal string output
+    if repr_meth in ['__repr__', '__str__'] and last_repr:
+        captured_std = u"{0}\n{1}".format(captured_std, last_repr)
+    if captured_std and not captured_std.isspace():
+        captured_std = CODE_OUTPUT.format(indent(captured_std, u' ' * 4))
+    else:
+        captured_std = ''
+    # give html output its own header
+    if repr_meth == '_repr_html_':
+        captured_html = HTML_HEADER.format(indent(last_repr, u' ' * 4))
+    else:
+        captured_html = ''
+
+    code_output = u"\n{0}\n\n{1}\n{2}\n\n".format(
+        images_rst, captured_std, captured_html
+    )
+    return code_output
+
+
+def _reset_cwd_syspath(cwd, sys_path):
+    """Reset cwd and sys.path."""
+    os.chdir(cwd)
+    sys.path = sys_path
+
+
+def execute_code_block(compiler, block, example_globals, script_vars,
+                       gallery_conf, file_conf):
+    """Execute the code block of the example file.
+
+    Parameters
+    ----------
+    compiler : codeop.Compile
+        Compiler to compile AST of code block.
+
+    block : List[Tuple[str, str, int]]
+        List of Tuples, each Tuple contains label ('text' or 'code'),
+        the corresponding content string of block and the leading line number.
+
+    example_globals: Dict[str, Any]
+        Global variables for examples.
+
+    script_vars : Dict[str, Any]
+        Configuration and runtime variables.
+
+    gallery_conf : Dict[str, Any]
+        Gallery configurations.
+
+    file_conf : Dict[str, Any]
+        File-specific settings given in source file comments as:
+        ``# sphinx_gallery_<name> = <value>``.
+
+    Returns
+    -------
+    code_output : str
+        Output of executing code in rST.
+    """
+    if example_globals is None:  # testing shortcut
+        example_globals = script_vars['fake_main'].__dict__
+    blabel, bcontent, lineno = block
+    # If example is not suitable to run, skip executing its blocks
+    if not script_vars['execute_script'] or blabel == 'text':
+        return ''
+
+    cwd = os.getcwd()
+    # Redirect output to stdout
+    src_file = script_vars['src_file']
+    logging_tee = _check_reset_logging_tee(src_file)
+    assert isinstance(logging_tee, _LoggingTee)
+
+    # First cd in the original example dir, so that any file
+    # created by the example get created in this directory
+    os.chdir(os.path.dirname(src_file))
+
+    sys_path = copy.deepcopy(sys.path)
+    sys.path.append(os.getcwd())
+
+    # Save figures unless there is a `sphinx_gallery_defer_figures` flag
+    match = re.search(r'^[\ \t]*#\s*sphinx_gallery_defer_figures[\ \t]*\n?',
+                      bcontent, re.MULTILINE)
+    need_save_figures = match is None
+
+    try:
+        ast_Module = _ast_module()
+        code_ast = ast_Module([bcontent])
+        flags = ast.PyCF_ONLY_AST | compiler.flags
+        code_ast = compile(bcontent, src_file, 'exec', flags, dont_inherit=1)
+        ast.increment_lineno(code_ast, lineno - 1)
+
+        is_last_expr, mem_max = _exec_and_get_memory(
+            compiler, ast_Module, code_ast, gallery_conf, script_vars
+        )
+        script_vars['memory_delta'].append(mem_max)
+        # This should be inside the try block, e.g., in case of a savefig error
+        logging_tee.restore_std()
+        if need_save_figures:
+            need_save_figures = False
+            images_rst = save_figures(block, script_vars, gallery_conf)
+        else:
+            images_rst = u''
+    except Exception:
+        logging_tee.restore_std()
+        except_rst = handle_exception(
+            sys.exc_info(), src_file, script_vars, gallery_conf
+        )
+        code_output = u"\n{0}\n\n\n\n".format(except_rst)
+        # still call this even though we won't use the images so that
+        # figures are closed
+        if need_save_figures:
+            save_figures(block, script_vars, gallery_conf)
+    else:
+        _reset_cwd_syspath(cwd, sys_path)
+
+        code_output = _get_code_output(
+            is_last_expr, example_globals, gallery_conf, logging_tee,
+            images_rst, file_conf
+        )
+    finally:
+        _reset_cwd_syspath(cwd, sys_path)
+        logging_tee.restore_std()
+
+    # Sanitize ANSI escape characters from RST output
+    ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
+    code_output = ansi_escape.sub('', code_output)
+
+    return code_output
+
+
 def executable_script(src_file, gallery_conf):
     """Validate if script has to be run according to gallery configuration
 
@@ -716,7 +888,7 @@ def _check_input(prompt=None):
         'Cannot use input() builtin function in Sphinx-Gallery examples')
 
 
-def execute_script(script_blocks, script_vars, gallery_conf):
+def execute_script(script_blocks, script_vars, gallery_conf, file_conf):
     """Execute and capture output from python script already in block structure
 
     Parameters
@@ -729,6 +901,9 @@ def execute_script(script_blocks, script_vars, gallery_conf):
         Configuration and run time variables
     gallery_conf : dict
         Contains the configuration of Sphinx-Gallery
+    file_conf : dict
+        File-specific settings given in source file comments as:
+        ``# sphinx_gallery_<name> = <value>``
 
     Returns
     -------
@@ -782,7 +957,8 @@ def execute_script(script_blocks, script_vars, gallery_conf):
         for block in script_blocks:
             logging_tee.set_std_and_reset_position()
             output_blocks.append(execute_code_block(
-                compiler, block, example_globals, script_vars, gallery_conf))
+                compiler, block, example_globals, script_vars, gallery_conf,
+                file_conf))
     time_elapsed = time() - t_start
     sys.argv = argv_orig
     script_vars['memory_delta'] = max(script_vars['memory_delta'])
@@ -857,20 +1033,44 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
         'src_file': src_file,
         'target_file': target_file}
 
-    if executable:
-        clean_modules(gallery_conf, fname)
+    if (
+        executable
+        and gallery_conf['reset_modules_order'] in ['before', 'both']
+    ):
+        clean_modules(gallery_conf, fname, 'before')
     output_blocks, time_elapsed = execute_script(script_blocks,
                                                  script_vars,
-                                                 gallery_conf)
+                                                 gallery_conf,
+                                                 file_conf)
 
     logger.debug("%s ran in : %.2g seconds\n", src_file, time_elapsed)
 
+    # Create dummy images
+    if not executable:
+        dummy_image = file_conf.get('dummy_images', None)
+        if dummy_image is not None:
+            if type(dummy_image) is not int:
+                raise ExtensionError(
+                    'sphinx_gallery_dummy_images setting is not a number, '
+                    'got %r' % (dummy_image,))
+
+            image_path_iterator = script_vars['image_path_iterator']
+            stock_img = os.path.join(glr_path_static(), 'no_image.png')
+            for _, path in zip(range(dummy_image), image_path_iterator):
+                if not os.path.isfile(path):
+                    copyfile(stock_img, path)
+
     if gallery_conf['remove_config_comments']:
         script_blocks = [
             (label, remove_config_comments(content), line_number)
             for label, content, line_number in script_blocks
         ]
 
+    script_blocks = [
+        (label, remove_ignore_blocks(content), line_number)
+        for label, content, line_number in script_blocks
+    ]
+
     # Remove final empty block, which can occur after config comments
     # are removed
     if script_blocks[-1][1].isspace():
@@ -904,14 +1104,26 @@ def generate_file_rst(fname, target_dir, src_dir, gallery_conf,
         with open(codeobj_fname, 'wb') as fid:
             pickle.dump(example_code_obj, fid, pickle.HIGHEST_PROTOCOL)
         _replace_md5(codeobj_fname)
-    backrefs = set('{module_short}.{name}'.format(**cobj)
-                   for cobjs in example_code_obj.values()
-                   for cobj in cobjs
-                   if cobj['module'].startswith(gallery_conf['doc_module']))
+    exclude_regex = gallery_conf['exclude_implicit_doc_regex']
+    backrefs = set(
+        '{module_short}.{name}'.format(**cobj)
+        for cobjs in example_code_obj.values()
+        for cobj in cobjs
+        if cobj['module'].startswith(gallery_conf['doc_module']) and (
+            cobj['is_explicit'] or
+            (not exclude_regex) or
+            (not exclude_regex.search('{module}.{name}'.format(**cobj)))
+        )
+    )
+
     # Write backreferences
     _write_backreferences(backrefs, seen_backrefs, gallery_conf, target_dir,
                           fname, intro, title)
 
+    del script_vars, global_variables  # don't keep these during reset
+    if executable and gallery_conf['reset_modules_order'] in ['after', 'both']:
+        clean_modules(gallery_conf, fname, 'after')
+
     return intro, title, (time_elapsed, memory_used)
 
 
@@ -996,6 +1208,7 @@ def rst_blocks(script_blocks, output_blocks, file_conf, gallery_conf):
         else:
             block_separator = '\n\n' if not bcontent.endswith('\n') else '\n'
             example_rst += bcontent + block_separator
+
     return example_rst
 
 
@@ -1039,13 +1252,15 @@ def save_rst_example(example_rst, example_file, time_elapsed,
     if len(binder_conf) > 0:
         binder_badge_rst += gen_binder_rst(example_file, binder_conf,
                                            gallery_conf)
+        binder_badge_rst = indent(binder_badge_rst, '  ')  # need an extra two
 
     fname = os.path.basename(example_file)
     example_rst += CODE_DOWNLOAD.format(fname,
                                         replace_py_ipynb(fname),
                                         binder_badge_rst,
                                         ref_fname)
-    example_rst += SPHX_GLR_SIG
+    if gallery_conf['show_signature']:
+        example_rst += SPHX_GLR_SIG
 
     write_file_new = re.sub(r'\.py$', '.rst.new', example_file)
     with codecs.open(write_file_new, 'w', encoding="utf-8") as f:
